{--------------------------------------------------------}
{                                                        }
{    Grafivox - gerador de grficos para cegos           }
{                                                        }
{    Mdulo de controle da impresso comum               }
{                                                        }
{    Autor:  Jos Antonio Borges                         }
{                                                        }
{    Em agosto/2018                                      }
{                                                        }
{--------------------------------------------------------}

unit gfimprime;

interface
uses
    dvcrt, dvwin, dvform, classes,
    windows, messages, graphics, sysutils, printers, WinSpool,
    gfvars, gfmsg, gfArq, gfConfPrn, gfDesen;

function escolheImpr: string;
procedure imprimeTinta (demo: boolean);

implementation

{--------------------------------------------------------}
{             imprime canvas na impressora               }
{--------------------------------------------------------}

function escolheImpr: string;
var
    impressoras: TStringList;
    i: integer;

begin
    result := '';
    impressoras := sintItensAmbienteArq('Devices', 'win.ini');
    popupMenuCria(whereX, whereY, 60, impressoras.Count, RED);

    for i := 0 to impressoras.Count-1 do
      popupMenuAdiciona('', impressoras[i]);
    i := popupMenuSeleciona;

    if i > 0 then
        begin
            result := impressoras[i-1];
(*
            GetProfileString('Devices', pchar (impressoras[i-1]), '', paramImp, 200);
            strcopy (padrao, pchar(impressoras[i-1]));
            strcat (padrao, ',');
            strcat (padrao, paramImp);
            writeProfileString ('windows', 'device', padrao);
*)
      end;

    impressoras.Free;
end;

{--------------------------------------------------------}
{             imprime canvas na impressora.              }
{--------------------------------------------------------}

procedure imprimeCanvas (bmp: TBitmap; ampl: real);
var
    Info: PBitmapInfo;
    InfoSize: DWORD;
    Image: Pointer;
    ImageSize: DWORD;
    Bits: HBITMAP;
    DIBWidth, DIBHeight: LongInt;

begin
  Printer.BeginDoc;
  try
      { Paint bitmap to the printer }
      with Printer do
      begin
        Bits := bmp.Handle;
        GetDIBSizes(Bits, InfoSize, ImageSize);
        Info := AllocMem(InfoSize);
        try
          Image := AllocMem(ImageSize);
          try
            GetDIB(Bits, 0, Info^, Image^);
            with Info^.bmiHeader do
            begin
              DIBWidth := biWidth;
              DIBHeight := biHeight;
            end;
            StretchDIBits(Canvas.Handle, 0, 0, xMaxPrinter, yMaxPrinter,
                      0, round (DIBHeight * (1.0-ampl)),
                      round (DIBWidth * ampl), round (DIBHeight * ampl),
                      Image, Info^, DIB_RGB_COLORS, SRCCOPY)
          finally
            FreeMem(Image, ImageSize);
          end;
        finally
          FreeMem(Info, InfoSize);
        end;
      end;
  finally
    Printer.EndDoc;
  end;
end;

{--------------------------------------------------------}
{                     imprime em tinta                   }
{--------------------------------------------------------}

procedure imprimeTinta (demo: boolean);
var
    nomeImpressora: string;
    PrinterToUse : integer;
begin
    mensagem ('GFSTINTA', 1);  {'Selecione a impressora a tinta'}

    nomeImpressora := escolheImpr;
    if nomeImpressora = '' then
        exit;

    PrinterToUse := Printer.Printers.IndexOf(nomeImpressora);
    if PrinterToUse > -1 then
        Printer.PrinterIndex :=  PrinterToUse;

    writeln;
    mensagem ('GFPDEITA', 0);    {'Imprime com papel deitado'}
    if popupMenuPorLetra ('SN') = 'S' then
        Printer.Orientation := poLandscape
    else
        Printer.Orientation := poPortrait;

    writeln;
    mensagem ('GFIMPREM', 0);   {'Imprimindo em '}
    sintwriteln (nomeImpressora);
    mensagem ('GFTAMPAP', 0);   {'Tamanho do papel (em pixels): '}
    sintwriteln (IntToStr (printer.PageWidth) +  ' x '                   +
                 IntToStr (printer.PageHeight));

    if printer.PageWidth < printer.PageHeight then
        xMaxPrinter := printer.PageWidth
    else
        xMaxPrinter := printer.PageHeight;
    yMaxPrinter := xMaxPrinter;

    saidaGrafica := paraTinta;
    criaBitmapParaDesenho;

    if demo then
        produzOGraficoDemo
    else
        produzOGrafico;

    imprimeCanvas (FGrafico, 1);

    FGrafico.Free;
end;

end.

