{--------------------------------------------------------}
{                                                        }
{    Grafivox - gerador de grficos para cegos           }
{                                                        }
{    Mdulo principal                                    }
{                                                        }
{    Autores: Jos Antonio Borges e Julio Tadeu Silveira }
{                                                        }
{    Em agosto/2018                                      }
{                                                        }
{--------------------------------------------------------}

program grafivox;

uses
  dvwin,
  dvcrt,
  dvdigiTexto,
  dvform,
  dvarq,
  windows,
  sysutils,
  classes,
  graphics,
  gfvars,
  gfmsg,
  gfbraille,
  gfdesen,
  gfintera,
  gfimprime,
  gfarq,
  gfconfig,
  gfconfprn,
  gftexto,
  gfequacao,
  gfgeraBrl,
  gfbrunif,
  gfDados;

{--------------------------------------------------------}
{                    Inicializa                          }
{--------------------------------------------------------}

function inicializa: boolean;
var ambiente: string;
begin
    result := True;
    banner;
    tituloJanela ('');

    ambiente := sintambiente('GRAFIVOX', 'DIRGRAFIVOX');
    if ambiente = '' then
        ambiente := sintAmbiente('DOSVOX', 'PGMDOSVOX') + '\som\grafivox';
    sintinic (0, ambiente);

    textBackground (BLUE);
    mensagem ('GFINIC', 0);   {'Grafivox - verso '}
    sintWriteln (versao);
    writeln;
    textBackground (BLACK);

    listaComandos := TStringList.Create;

    nomeArq := pegaNomeArq;
    if teclaObtemNomeArq = ESC then
        begin
            mensagem ('GFDESIST', 1);  {'Desistiu'}
            result := false;
            exit;
        end;

    if nomeArq <> '' then
        carregaArquivo;
end;

{--------------------------------------------------------}
{                      Finaliza                          }
{--------------------------------------------------------}

procedure finaliza;
begin
    if listaComandos <> NIL then
        listaComandos.Free;
    mensagem ('GFFIM', 1);   {'Fim do Grafivox'}
    sintfim;
end;

{--------------------------------------------------------}
{                        ajuda
{--------------------------------------------------------}

procedure ajudaOpcao;
begin
    writeln;
    mensagem ('GFOPCAO', 1);  {'As opes so:'}
    mensagem ('GFOPC_P', 1);  {'P - passo-a-passo interativo'}
    mensagem ('GFOPC_E', 1);  {'E - editar os comandos para graficar'}
    mensagem ('GFOPC_V', 1);  {'V - visualizar na tela'}
    mensagem ('GFOPC_S', 1);  {'S - salvar os comandos grficos'}
    mensagem ('GFOPC_B', 1);  {'B - imprimir em Braille'}
    mensagem ('GFOPC_N', 1);  {'N - novo grfico'}
    mensagem ('GFOPC_I', 1);  {'I - imprimir em tinta'}
    mensagem ('GFOPC_G', 1);  {'G - gravar imagem do grfico'}
    mensagem ('GFOPC_C', 1);  {'C - configurar'}
    mensagem ('GF_ESC',  1);  {'ESC - terminar'}

    while keypressed do readkey;
    tituloJanela(nomeArq);
    limpaBaixo(3);
    sintBip;
end;

{--------------------------------------------------------}
{            seleciona a opo com as setas
{--------------------------------------------------------}

function selSetasOpcao: char;

    procedure MenuAdiciona (msg: string);
    begin
         popupMenuAdiciona (msg, pegaTextoMensagem (msg));
    end;

const
    tabLetrasOpcao: string = 'PEVSBIGCN' + ESC;
var
    n: integer;
    salvaY: integer;

begin
    salvaY := whereY;
    if wherex <> 1 then writeln;
    sintwriteln (pegaTextoMensagem('GFOPCAO'));  {'As opes so:'}

    garanteEspacoTela (length(TabLetrasOpcao));
    popupMenuCria (wherex, wherey, 50, length(tabLetrasOpcao), MAGENTA);
    menuAdiciona ('GFOPC_P');  {'P - passo-a-passo interativo'}
    menuAdiciona ('GFOPC_E');  {'E - editar os comandos para graficar'}
    menuAdiciona ('GFOPC_V');  {'V - visualizar'}
    menuAdiciona ('GFOPC_S');  {'S - salvar os comandos grficos'}
    menuAdiciona ('GFOPC_B');  {'B - imprimir em Braille'}
    menuAdiciona ('GFOPC_I');  {'I - imprimir em tinta'}
    menuAdiciona ('GFOPC_G');  {'G - gravar imagem do grfico'}
    menuAdiciona ('GFOPC_C');  {'C - configurar'}
    menuAdiciona ('GFOPC_N');  {'N - novo grfico'}
    menuAdiciona ('GF_ESC');   {'ESC - terminar'}

    n := popupMenuSeleciona;
    if n > 0 then
        begin
            selSetasOpcao := tabLetrasOpcao[n];
            writeln (tabLetrasOpcao[n]);
        end
    else
        selSetasOpcao := ESC;
    limpaBaixo (salvaY);
end;

{--------------------------------------------------------}
{                editorao dos comandos                 }
{--------------------------------------------------------}

procedure editaComandos;
begin
    textBackground (RED);
    mensagem ('GFEDIT', 1);   {Editando}
    textBackground (BLACK);

    dvdigitexto.popupDigiTexto (listaComandos, False, False,
                                wherex, wherey, 80, 25-wherey, False);
end;

{--------------------------------------------------------}
{                      Processa                          }
{--------------------------------------------------------}

procedure processa;
var c, c2: char;
    opcao: string;
label fim;

begin
    saidaGrafica := paraTela;
    while true do
        begin
            tituloJanela(nomeArq);
            banner;

            textBackground (BLUE);
            mensagem ('GFOQUE', 0);      {'Grafivox - Que deseja? '}
            textBackground (BLACK);
            sintLeTecla (c, c2);
            opcao := '';
            writeln;

            if (c = #0) and ((c2 = CIMA) or (c2 = BAIX) or (c2 = F9)) then
                  begin
                      c := selSetasOpcao;
                      if c <> #$1b then
                          opcao := copy (
                             opcoesItemSelecionado, pos ('-', opcoesItemSelecionado)-1, 999);
                  end;

            if c = #$1b then
                begin
                    writeln;
                    goto fim;
                end;

            if (c = #$0d) then continue
            else
            if (c = #0) and (c2 = F1) then
                 ajudaOpcao
            else
                begin
                    case upcase(c) of

                        'E': editaComandos;
                        'P': passoAPasso;
                        'S': salvaArquivo;
                        'N': novoGrafico;

                        'V': visualiza (False);
                         ^V: visualiza (True);

                        'B': imprimeBraille (False);
                         ^B: imprimeBraille (True);

                        'I': imprimeTinta (False);
                         ^I: imprimeTinta (True);

                        'G': gravaArquivoGrafico;

                        'C': configura;

                        ESC: ;

                    else
                        mensagem ('GFOPINV', 1);    {'Opo invlida'}
                    end;
                end;
        end;
fim:
    writeln;
end;

{--------------------------------------------------------}
{              verifica se gravao pendente             }
{--------------------------------------------------------}

function veSeTermina: boolean;
begin
    mensagem ('GFCNFFIM', 0);  {'Confirma o fim do grafivox? '}
    if popupMenuPorLetra('SN') <> 'S' then
        begin
            mensagem ('GFCANC', 2);   {'Cancelado'}
            result := false;
            exit;
        end;

    if gravacaoPendente then
        begin
            mensagem ('GFNAOGV', 1);  {'O projeto ainda no foi salvo.'}
            mensagem ('GFCNFNGV', 0); {'Confirma que no quer gravar? '}
            result := popupMenuPorLetra('SN') = 'S';
        end
    else
        result := true;
end;

{--------------------------------------------------------}
{                   Programa principal                   }
{--------------------------------------------------------}

begin
    if inicializa then
        repeat
            processa;
        until veSeTermina;

    finaliza;
end.
